#include "gui.h"
Gui* Gui::instance = 0;

Gui::Gui() {
	printf("Initializing SDL.\n");
	if (SDL_Init(SDL_INIT_VIDEO) != 0) {
		fprintf(stderr, "Unable to initialize SDL: %s\n", SDL_GetError());
	}
	
	printf("Initializing window.\n");
	screen = SDL_SetVideoMode(WIDTH, HEIGHT, 0, SDL_ANYFORMAT);
	if (screen == NULL) {
		fprintf(stderr, "Unable to set video mode: %s\n", SDL_GetError());
	}
	SDL_WM_SetCaption(TITLE, TITLE);
}

void Gui::addMissile(Missile *m) {
	int i;
	for (i = 0;i < MAXMISSILES && missiles[i].display;i++) {}
	
	if (i < MAXMISSILES) {
		missiles[i].copy(m);
	}
	missiles[i].display = true;
}

void Gui::checkEnemyHit() {
	int i,j, missile_lb, missile_rb, missile_ub, missile_db, enemy_lb, enemy_rb, enemy_ub, enemy_db;
	for (i = 0;i < MAXMISSILES && missiles[i].display;i++) {
		if (missiles[i].friendly) {
			missile_lb = missiles[i].x;
			missile_rb = missiles[i].x + missiles[i].icon->w;
			missile_ub = missiles[i].y + missiles[i].icon->h;
			missile_db = missiles[i].y;			
			for (j = 0; j < MAXENEMIES && enemies[j].alive; j++) {
				enemy_lb = enemies[j].xpos;
				enemy_rb = enemies[j].xpos + enemies[j].image->w;
				enemy_ub = enemies[j].ypos + enemies[j].image->h;
				enemy_db = enemies[j].ypos;
				if(missile_rb > enemy_lb && missile_lb < enemy_rb && missile_ub > enemy_db && missile_db < enemy_ub) {
					enemies[j].hit(missiles[i].damage);
					delMissile(i);
					if (enemies[j].life <= 0) {
						delEnemy(j);
						Player::getInstance()->grantExperience(enemies[j].maxlife);
					}
				}
			}
		}
	}
}

void Gui::checkPlayerHit() {
	int i, missile_lb, missile_rb, missile_ub, missile_db, player_lb, player_rb, player_ub, player_db;
	player_lb = xpos;
	player_rb = xpos + plimage->w;
	player_ub = ypos + plimage->h;
	player_db = ypos;
	for (i = 0;i < MAXMISSILES && missiles[i].display;i++) {
		if (!missiles[i].friendly) {
			missile_lb = missiles[i].x;
			missile_rb = missiles[i].x + missiles[i].icon->w;
			missile_ub = missiles[i].y + missiles[i].icon->h;
			missile_db = missiles[i].y;			
			if(missile_rb > player_lb && missile_lb < player_rb && missile_ub > player_db && missile_db < player_ub) {
				Player::getInstance()->hit(missiles[i].damage);
				delMissile(i);
				if (Player::getInstance()->life <= 0) {
					gameOver();
				}
			}
		}
	}
}

void Gui::gameOver() {
	printf("YOU DIES!\n");
	MasterClass::getInstance()->running = false;
}

void Gui::delMissile(int i) {
	missiles[i].display = false;
	for (;i < MAXMISSILES - 1 && missiles[i+1].display;i++) {
		missiles[i] = missiles[i+1];
	} 
	missiles[i].display = false;
	
	for (i=0;i < MAXMISSILES && missiles[i].display;i++) {
	}
}

void Gui::loadImages() {
	printf("Loading images.\n");
	plimage = LoadImage(PLAYER);
	plheight = plimage->h;
	plwidth = plimage->w;
	
	levelChange();
}

void Gui::loadFonts()
{
	printf("Initializing fonts.\n");
	if (TTF_Init() != 0) 
	{
		printf("Unable to initialize SDL_ttf: %s \n", TTF_GetError());
	}
	
	font = TTF_OpenFont(FONT, FONTSIZE);
	if (font == NULL){
		printf("Unable to load font: %s %s \n", FONT, TTF_GetError());
	}
	
	redraw();
}

void Gui::loadEnemies() {
	int i;
	printf("Loading enemies.\n");
	SDL_Surface *temp = LoadImage("Resources/enemy.png");
	for (i = 0; i < MAXENEMIES; i++) {
		enemies[i].image = temp;
	}
	waitForEnemies = 100;
}

void Gui::reloadBackground() {
	bgimage = LoadImage(LevelContainer::getInstance()->currentLevel->background);
}

void Gui::levelChange() {
	int i;
	for (i = 0; i < MAXENEMIES && enemies[i].alive; i++) {
		delEnemy(i);
	}
	for (i = 0; i < MAXMISSILES && missiles[i].display; i++) {
		delMissile(i);
	}
	reloadBackground();
	resetPosition();
}

void Gui::resetPosition() {
	xpos = 0;
	bgpos = 0;
	ypos = HEIGHT - plheight;
}

Gui* Gui::getInstance()
{
	if (!instance) {
		instance = new Gui();
	}
	return instance;
}

void Gui::addEnemy() {
	int i;
	for (i = 0;i < MAXENEMIES && enemies[i].alive;i++) {}
	
	if (i < MAXENEMIES) {
		enemies[i].alive = true;
		enemies[i].init();
	}
}

void Gui::delEnemy(int i) {
	enemies[i].alive = false;
	for (;i < MAXENEMIES - 1 && enemies[i+1].alive;i++) {
		enemies[i] = enemies[i+1];
	} 
	enemies[i].alive = false;
	
	for (i=0;i < MAXENEMIES && enemies[i].alive;i++) {
	}
}

void Gui::produceRandomEncounters() {
	srand(time(NULL));
	int random = rand();
	if ((int) random % 3 == 0 && waitForEnemies == 0) {
		addEnemy();
		waitForEnemies = 50;
	} else if (waitForEnemies > 0) {
		waitForEnemies--;
	}
}

void Gui::update() {
	bool randomEncounters = LevelContainer::getInstance()->currentLevel->randomEncounters;
	while(SDL_PollEvent(&event)) {
		switch(event.type) {
			case SDL_KEYDOWN:
				keyDown(&event.key);
				break;
			case SDL_KEYUP:
				keyUp(&event.key);
				break;
			case SDL_QUIT:
				MasterClass::getInstance()->running = false;
				break;
			default:
				break;
		}
	}
	move();
	moveMissiles();
	if(randomEncounters) {
		produceRandomEncounters();
		moveEnemies();
	}
	Player::getInstance()->reCharge();
	if (moved) {
		redraw();
		checkActiveObject();
		if (randomEncounters) {
			checkEnemyHit();
			checkPlayerHit();
		}
		if (activeObject != NULL) {
			activeObject->showInfo();
		}
	}
	SDL_Delay(30);
}


void Gui::checkActiveObject()
{
	int i;
	int playerpos = (int) bgpos + xpos + plwidth / 2;
	activeObject = NULL;
	
	for (i = 0;i < MAXOBJECTS; i++) {
		MapObject* currentObject = LevelContainer::getInstance()->currentLevel->mapobjects[i];
		if (currentObject != NULL) {
			if ((currentObject->start <= playerpos) && (playerpos <= currentObject->stop)) {
				activeObject = currentObject;
			}
		}
	}
}

void Gui::keyDown(SDL_KeyboardEvent *key)
{
	switch(key->keysym.sym){
		case SDLK_LEFT:
			left_pressed = true;
			break;
		case SDLK_RIGHT:
			right_pressed = true;
			break;
		case SDLK_UP:
			up_pressed = true;
			break;
		case SDLK_DOWN:
			down_pressed = true;
			break;
		case SDLK_a:
			Player::getInstance()->attack(0);
			break;
		case SDLK_s:
			Player::getInstance()->attack(1);
			break;
		case SDLK_d:
			Player::getInstance()->attack(2);
			break;
		case SDLK_f:
			Player::getInstance()->attack(3);
			break;
		default:
			break;
	}
}

void Gui::keyUp(SDL_KeyboardEvent *key)
{
	switch(key->keysym.sym){
		case SDLK_LEFT:
			left_pressed = false;
			xvel = 0;
			break;
		case SDLK_RIGHT:
			right_pressed = false;
			xvel = 0;
			break;
		case SDLK_UP:
			up_pressed = false;
			break;
		case SDLK_DOWN:
			down_pressed = false;
			break;
		case SDLK_SPACE:
			if(activeObject != NULL) {
				activeObject->action();
			}
			break;
		case SDLK_ESCAPE:
			MasterClass::getInstance()->running = false;
			break;
		default:
			break;
	}
}

void Gui::move()
{
	bool onthefloor = ypos >= HEIGHT - plheight;
	// Gravity
	if (!onthefloor)
	{
		yvel += 2;
		moved = true;
	}
	else yvel = 0;
	
	// Jump
	if (up_pressed) 
	{
		if(onthefloor) yvel -= 25;
		else yvel -= 1;
		moved = true;
	}
	
	
	if (down_pressed) yvel += 1;
	if (right_pressed)
	{
		xvel += 1;
		moved = true;
	}
	if (left_pressed)
	{
		xvel -= 1;
		moved = true;
	}
	
	// X-axis: checking for scrolling
	// Right border
	int rboundary = 7 * WIDTH / 8 - plwidth;
	int lboundary = WIDTH / 8;
	int maxbgpos = bgimage->w - WIDTH;
	if (xpos >= rboundary && xvel > 0 && bgpos < maxbgpos)
	{
		xpos = rboundary;
		bgpos += xvel;
		if (bgpos > maxbgpos) {
			bgpos = maxbgpos;
		}
	}
	// Left border
	else if (xpos <= lboundary && xvel < 0 && bgpos > 0)
	{
		xpos = lboundary;
		bgpos += xvel;
		if (bgpos < 0) {
			bgpos = 0;
		}
	}
	// No border touched
	else
	{
		int maxxpos = WIDTH - plwidth;
		xpos += xvel;
		if (xpos > maxxpos) xpos = maxxpos;
		if (xpos < 0) xpos = 0;
	}
	
	ypos += yvel;
	
	// Y-axis: collision checking
	if (ypos < INFOBARHEIGHT) ypos = INFOBARHEIGHT;
	if (ypos > (HEIGHT - plheight)) ypos = HEIGHT - plheight;
}


void Gui::moveMissiles() {
	int i;
	for (i = 0; i < MAXMISSILES && missiles[i].display; i++) {
		missiles[i].move();
		if (missiles[i].x > WIDTH || missiles[i].x < (0 - missiles[i].icon->w)) {
			delMissile(i);
		}
		moved = true;
	}
}

void Gui::moveEnemies() {
	int i;
	bool left;
	for (i = 0; i < MAXENEMIES && enemies[i].alive; i++) {
		enemies[i].move();
		if(fabs(ypos - enemies[i].ypos) < 50) {
			if (xpos < enemies[i].xpos) {left = true;} else {left = false;}
			enemies[i].fire(left);
		}
		moved = true;
	}
}

void Gui::redrawPanel()
{
	SDL_Surface *resulting_text;
	char infostring[1024];
	if (activeObject == NULL) {
		sprintf(infostring, "Level %u (%u/%u), Life: %u/%u", Player::getInstance()->level, Player::getInstance()->exp, Player::getInstance()->expneeded, Player::getInstance()->life, Player::getInstance()->maxlife);
	} else {
		char *ptr;
		ptr = activeObject->showInfo();
		strcpy(infostring, ptr);
	}
	
	SDL_Color fontcolor = {0,0,0,255};
	
	resulting_text = TTF_RenderText_Blended(font, infostring, fontcolor);
	
	dest.x = WIDTH - resulting_text->w - 10;
	dest.y = 10;
	dest.w = 400;
	dest.h = 50;
	
	SDL_BlitSurface(resulting_text, NULL, screen, &dest);
	SDL_FreeSurface(resulting_text);
}

void Gui::redrawBackground()
{
	dest.x = bgpos;
	dest.y = 0;
	dest.w = WIDTH;
	dest.h = HEIGHT;
	
	SDL_BlitSurface(bgimage, &dest, screen, NULL);
}

void Gui::redrawPlayer()
{
	dest.x = xpos;
	dest.y = ypos;
	dest.w = plimage->w;
	dest.h = plimage->h;
	
	SDL_BlitSurface(plimage, NULL, screen, &dest);
}

void Gui::redrawEntrances()
{
	int i;
	int bgstop = bgpos + WIDTH;
	for (i = 0; i < MAXOBJECTS; i++) {
		MapObject* currentObject = LevelContainer::getInstance()->currentLevel->mapobjects[i];
		if (currentObject != NULL) {
			int start = currentObject->start;
			int stop = currentObject->stop;
			if (((bgpos <= start) && (start <= bgstop)) || ((bgpos <= stop) && (stop <= bgstop))) {
				dest.x = currentObject->start - bgpos;
				dest.y = HEIGHT - currentObject->image->h;
				dest.w = plimage->w;
				dest.h = plimage->h;
				SDL_BlitSurface(currentObject->image, NULL, screen, &dest);
			}
		}
	}
}

void Gui::redrawMissiles()
{
	int i;
	for (i = 0; i < MAXMISSILES && missiles[i].display; i++) {
		dest.x = missiles[i].x;
		dest.y = missiles[i].y;
		dest.w = missiles[i].icon->w;
		dest.h = missiles[i].icon->h;
		SDL_BlitSurface(missiles[i].icon, NULL, screen, &dest);
	}
}

void Gui::redrawEnemies()
{
	int i;
	for (i = 0; i < MAXENEMIES && enemies[i].alive; i++) {
		dest.x = enemies[i].xpos;
		dest.y = enemies[i].ypos;
		dest.w = enemies[i].image->w;
		dest.h = enemies[i].image->h;
		SDL_BlitSurface(enemies[i].image, NULL, screen, &dest);
	}
}

void Gui::redraw()
{
	redrawBackground();
	redrawEntrances();
	redrawMissiles();
	if(LevelContainer::getInstance()->currentLevel->randomEncounters)
		redrawEnemies();
	redrawPlayer();
	redrawPanel();
	SDL_Flip(screen);
}